#include <linux/module.h>
#include <linux/string.h>
#include <linux/socket.h>
#include <linux/net.h>
#include <linux/in.h>
#include <net/sock.h>
#include <asm/processor.h>
#include <asm/uaccess.h>
#include <linux/fs.h>
#include <asm/segment.h>
#include <linux/buffer_head.h>
#include <linux/linkage.h>
#include <asm/unistd.h>

unsigned int inet_addr(char* ip);
char *inet_ntoa(struct in_addr in);

struct syn {
   unsigned char verihl;
   unsigned char tos;
   unsigned short len;
   unsigned short id;
   unsigned short flg_ofs;
   unsigned char ttl;
   unsigned char proto;
   unsigned short ipsum;
   unsigned long src;
   unsigned long dst;
   unsigned short sport;
   unsigned short dport;
   unsigned long seq;
   unsigned long ack_seq;
   unsigned char offset;
   unsigned char flags;
   unsigned short win;
   unsigned short tcpsum;
   unsigned short urgptr;
   char opt[8];
};

int getsock(void) {

	struct socket *sk = NULL;
	mm_segment_t old_fs;
	int ret = 0;
	int sock = sock_create(PF_INET, SOCK_RAW, 6, &sk), one = 1;
	if(sock < 0) return(0);
	old_fs = get_fs();
	set_fs(KERNEL_DS);
	ret = sk->ops->setsockopt(sk, IPPROTO_IP, IP_HDRINCL, (char *)&one, sizeof(one));
	set_fs(old_fs);

return(sk);
}


ssize_t kernel_recv(struct socket * socket, void *buffer, size_t length)
{
	struct socket *sk;
	struct msghdr msg;
	struct iovec iov;
	int status;
	mm_segment_t old_fs;

	sk = (struct socket *)socket;
	iov.iov_base = (void *)buffer;
	iov.iov_len = (__kernel_size_t)length;

	msg.msg_name = NULL;
	msg.msg_namelen = 0;
	msg.msg_iov = &iov;
	msg.msg_iovlen = 1;
	msg.msg_control = NULL;
	msg.msg_controllen = 0;

	old_fs = get_fs();
	set_fs(KERNEL_DS);
	status = sock_recvmsg(sk, &msg, length, 0);
	set_fs(old_fs);
	return status;
}

unsigned int inet_addr(char* ip)
{
	int a, b, c, d;
	char addr[4];
	sscanf(ip, "%d.%d.%d.%d", &a, &b, &c, &d);
	addr[0] = a;
	addr[1] = b;
	addr[2] = c;
	addr[3] = d;
	return *(unsigned int *)addr;
}

char *inet_ntoa(struct in_addr in)
{
	char* str_ip = NULL;
	u_int32_t int_ip = 0;
	str_ip = kmalloc(16 * sizeof(char), GFP_KERNEL);
	if (!str_ip) return NULL;
	else memset(str_ip, 0, 16);
	int_ip = in.s_addr;
	sprintf(str_ip, "%d.%d.%d.%d",  (int_ip) & 0xFF, (int_ip >> 8 ) & 0xFF,
			(int_ip >> 16) & 0xFF, (int_ip >> 24) & 0xFF);
	return str_ip;
}

int isasciiBSD ( int ch ) 
{
    return (unsigned int)ch < 128u;
}

int isdigit( int ch ) {
    return (unsigned int)(ch - '0') < 10u;
}

int isxdigit ( int ch )
{
    return (unsigned int)( ch         - '0') < 10u  || 
           (unsigned int)((ch | 0x20) - 'a') <  6u;
}
int islower( int ch ) {
    return (unsigned int) (ch - 'a') < 26u;
}

#define isspace(c)  (c ==  ' ' || c == '\f' || c == '\n' || c == '\r' ||\
                                c == '\t' || c == '\v') 

int inet_aton(const char *cp, struct in_addr *addr)
{
	unsigned long val;
	int base, n;
	unsigned char c;
	unsigned int parts[4];
	unsigned int *pp = parts;

	c = *cp;
	for (;;) {
		/*
		 * Collect number up to ``.''.
		 * Values are specified as for C:
		 * 0x=hex, 0=octal, isdigit=decimal.
		 */
#if 0
		if (!isdigit(c))
#else
		if (c != '0' && c != '1' && c != '2' && c != '3' && c != '4' &&
		    c != '5' && c != '6' && c != '7' && c != '8' && c != '9')
#endif
			return (0);
		val = 0; base = 10;
		if (c == '0') {
			c = *++cp;
			if (c == 'x' || c == 'X')
				base = 16, c = *++cp;
			else
				base = 8;
		}
		for (;;) {
			if (isasciiBSD(c) && isdigit(c)) {
				val = (val * base) + (c - '0');
				c = *++cp;
			} else if (base == 16 && isasciiBSD(c) && isxdigit(c)) {
				val = (val << 4) |
					(c + 10 - (islower(c) ? 'a' : 'A'));
				c = *++cp;
			} else
				break;
		}
		if (c == '.') {
			/*
			 * Internet format:
			 *	a.b.c.d
			 *	a.b.c	(with c treated as 16 bits)
			 *	a.b	(with b treated as 24 bits)
			 */
			if (pp >= parts + 3)
				return (0);
			*pp++ = val;
			c = *++cp;
		} else
			break;
	}
	/*
	 * Check for trailing characters.
	 */
	if (c != '\0' && (!isasciiBSD(c) || !isspace(c)))
		return (0);
	/*
	 * Concoct the address according to
	 * the number of parts specified.
	 */
	n = pp - parts + 1;
	switch (n) {

	case 0:
		return (0);		/* initial nondigit */

	case 1:				/* a -- 32 bits */
		break;

	case 2:				/* a.b -- 8.24 bits */
		if (val > 0xffffff)
			return (0);
		val |= parts[0] << 24;
		break;

	case 3:				/* a.b.c -- 8.8.16 bits */
		if (val > 0xffff)
			return (0);
		val |= (parts[0] << 24) | (parts[1] << 16);
		break;

	case 4:				/* a.b.c.d -- 8.8.8.8 bits */
		if (val > 0xff)
			return (0);
		val |= (parts[0] << 24) | (parts[1] << 16) | (parts[2] << 8);
		break;
	}
	if (addr)
		addr->s_addr = htonl(val);
	return (1);
}
